@@ -1,4 +1,15 @@
+#
+# This file is part of Const-Fast
+#
+# This software is copyright (c) 2010 by Leon Timmermans.
+#
+# This is free software; you can redistribute it and/or modify it under
+# the same terms as the Perl 5 programming language system itself.
+#
 package Const::Fast;
+BEGIN {
+  $Const::Fast::VERSION = '0.007';
+}
 
 use 5.008;
 use strict;
@@ -7,8 +18,7 @@ use warnings FATAL => 'all';
 use Scalar::Util qw/reftype blessed/;
 use Carp qw/croak/;
 use Sub::Exporter -setup => { exports => [qw/const/], groups => { default => [qw/const/] } };
-
-our $VERSION = '0.006';
+use Storable qw/dclone/;
 
 ## no critic (RequireArgUnpacking, ProhibitAmpersandSigils)
 # The use of $_[0] is deliberate and essential, to be able to use it as an lvalue and to keep the refcount down.
@@ -17,19 +27,15 @@ sub _make_readonly {
 	my (undef, $dont_clone) = @_;
 	if (my $reftype = reftype $_[0] and not blessed($_[0]) and not &Internals::SvREADONLY($_[0])) {
 		my $needs_cloning = !$dont_clone && &Internals::SvREFCNT($_[0]) > 1;
+		$_[0] = dclone($_[0]) if $needs_cloning;
 		&Internals::SvREADONLY($_[0], 1);
 		if ($reftype eq 'ARRAY') {
-			$_[0] = [ @{ $_[0] } ] if $needs_cloning;
 			_make_readonly($_) for @{ $_[0] };
 		}
 		elsif ($reftype eq 'HASH') {
-			$_[0] = { %{ $_[0] } } if $needs_cloning;
 			&Internals::hv_clear_placeholders($_[0]);
 			_make_readonly($_) for values %{ $_[0] };
 		}
-		elsif ($reftype eq 'SCALAR' and $needs_cloning) {
-			$_[0] = \(my $anon = ${ $_[0] });
-		}
 	}
 	Internals::SvREADONLY($_[0], 1);
 	return;
@@ -60,7 +66,11 @@ sub const(\[$@%]@) {
 
 1;    # End of Const::Fast
 
-__END__
+# ABSTRACT: Facility for creating read-only scalars, arrays, and hashes
+
+
+
+=pod
 
 =head1 NAME
 
@@ -68,7 +78,7 @@ Const::Fast - Facility for creating read-only scalars, arrays, and hashes
 
 =head1 VERSION
 
-Version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -92,58 +102,35 @@ Exporting is done using Sub::Exporter for flexibility on import.
 
 =head1 RATIONALE
 
-This module was written because I stumbled on some serious issues of L<Readonly> that aren't easily fixable without breaking backwards compatibility in subtle ways. In particular Readonly's use of ties is a source of subtle bugs and bad performance. Instead, this module uses the builtin readonly feature of perl, making access to the variables just as fast as any normal variable without the weird side-effects of ties. Readonly can do the same for scalars when Readonly::XS is installed, but chooses not to do so in the most common case.
+This module was written because I stumbled on some serious issues of L<Readonly> that aren't easily fixable without breaking backwards compatibility in subtle ways. In particular Readonly's use of ties is a source of subtle bugs and bad performance. Instead, this module uses the builtin readonly feature of perl, making access to the variables just as fast as any normal variable without the weird side-effects of ties. Readonly can do the same for scalars when Readonly::XS is installed, but chooses not to do so in the most common case. This may change in the future if someone takes up maintenance of Readonly, and the two modules may be convergence if that happens.
 
-=head1 AUTHOR
+=head1 CAVEATS
 
-Leon Timmermans, C<< <leont at cpan.org> >>
+Perl doesn't distinguish between restricted hashes and readonly hashes. This means that:
 
-=head1 BUGS
+ use Const::Fast;
+ const my %a => (foo => 1, bar => 2);
+ say 1 unless $a{baz}
 
-Please report any bugs or feature requests to C<bug-const-fast at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Const-Fast>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Const::Fast
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Const-Fast>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Const-Fast>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Const-Fast>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Const-Fast/>
-
-=back
+Will give the error "Attempt to access disallowed key 'baz' in a restricted hash". You have to use C<exists $a{baz}> instead. This is a limitation of perl that can hopefully be solved in the future.
 
 =head1 ACKNOWLEDGEMENTS
 
 The interface for this module was inspired by Eric Roode's L<Readonly>. The implementation is inspired by doing everything the opposite way Readonly does it.
 
-=head1 LICENSE AND COPYRIGHT
+=head1 AUTHOR
 
-Copyright 2010 Leon Timmermans.
+Leon Timmermans <fawaka@gmail.com>
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
+=head1 COPYRIGHT AND LICENSE
 
-See http://dev.perl.org/licenses/ for more information.
+This software is copyright (c) 2010 by Leon Timmermans.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+
+__END__
+
